@@ -10,12 +10,12 @@ pub use crate::{
     length::{LengthUnit, RsvgLength as Length},
 };
 
-use glib::prelude::*;
 use url::Url;
 
 use std::path::Path;
 
-use gio::{Cancellable, FileExt};
+use gio::prelude::*; // Re-exposes glib's prelude as well
+use gio::Cancellable;
 
 use crate::{
     dpi::Dpi,
@@ -23,16 +23,14 @@ use crate::{
     url_resolver::UrlResolver,
 };
 
-/// Builder for loading an [`SvgHandle`][SvgHandle].
+/// Builder for loading an [`SvgHandle`].
 ///
 /// This is the starting point for using librsvg.  This struct
-/// implements a builder pattern for configuring an
-/// [`SvgHandle`][SvgHandle]'s options, and then loading the SVG data.
-/// You can call the methods of `Loader` in sequence to configure
-/// how SVG data should be loaded, and finally use one of the loading
-/// functions to load an [`SvgHandle`][SvgHandle].
-///
-/// [SvgHandle]: struct.SvgHandle.html
+/// implements a builder pattern for configuring an [`SvgHandle`]'s
+/// options, and then loading the SVG data.  You can call the methods
+/// of `Loader` in sequence to configure how SVG data should be
+/// loaded, and finally use one of the loading functions to load an
+/// [`SvgHandle`].
 #[derive(Default)]
 pub struct Loader {
     unlimited_size: bool,
@@ -110,7 +108,7 @@ impl Loader {
     /// let mut output = env::temp_dir();
     /// output.push("output.pdf");
     /// let surface = cairo::PdfSurface::new(640.0, 480.0, output)?;
-    /// let cr = cairo::Context::new(&surface);
+    /// let cr = cairo::Context::new(&surface).expect("Failed to create a cairo context");
     ///
     /// let renderer = librsvg::CairoRenderer::new(&svg_handle);
     /// renderer.render_document(
@@ -134,7 +132,7 @@ impl Loader {
     ///     .unwrap();
     /// ```
     pub fn read_path<P: AsRef<Path>>(self, path: P) -> Result<SvgHandle, LoadingError> {
-        let file = gio::File::new_for_path(path);
+        let file = gio::File::for_path(path);
         self.read_file(&file, None::<&Cancellable>)
     }
 
@@ -145,7 +143,7 @@ impl Loader {
     /// # Example:
     /// ```
     /// let svg_handle = librsvg::Loader::new()
-    ///     .read_file(&gio::File::new_for_path("example.svg"), None::<&gio::Cancellable>)
+    ///     .read_file(&gio::File::for_path("example.svg"), None::<&gio::Cancellable>)
     ///     .unwrap();
     /// ```
     pub fn read_file<F: IsA<gio::File>, P: IsA<Cancellable>>(
@@ -159,9 +157,8 @@ impl Loader {
 
     /// Reads an SVG stream from a `gio::InputStream`.
     ///
-    /// This is similar to the [`read`](#method.read) method, but
-    /// takes a `gio::InputStream`.  The `base_file`, if it is not
-    /// `None`, is used to extract the base URL for this stream.
+    /// The `base_file`, if it is not `None`, is used to extract the
+    /// [base URL][crate#the-base-file-and-resolving-references-to-external-files] for this stream.
     ///
     /// Reading an SVG document may involve resolving relative URLs if the
     /// SVG references things like raster images, or other SVG files.
@@ -175,7 +172,7 @@ impl Loader {
     /// ```
     /// use gio::prelude::*;
     ///
-    /// let file = gio::File::new_for_path("example.svg");
+    /// let file = gio::File::for_path("example.svg");
     ///
     /// let stream = file.read(None::<&gio::Cancellable>).unwrap();
     ///
@@ -210,13 +207,13 @@ impl Loader {
 }
 
 fn url_from_file(file: &gio::File) -> Result<Url, LoadingError> {
-    Url::parse(&file.get_uri()).map_err(|_| LoadingError::BadUrl)
+    Url::parse(&file.uri()).map_err(|_| LoadingError::BadUrl)
 }
 
 /// Handle used to hold SVG data in memory.
 ///
 /// You can create this from one of the `read` methods in
-/// [`Loader`](#struct.Loader.html).
+/// [`Loader`].
 pub struct SvgHandle(Handle);
 
 impl SvgHandle {
@@ -270,8 +267,6 @@ const DEFAULT_DPI_Y: f64 = 96.0;
 /// <svg xmlns="http://www.w3.org/2000/svg" width="100" height="400">
 /// ```
 /// In this case, the length fields will be set to `Some()`, and `vbox` to `None`.
-///
-/// [`CairoRenderer::intrinsic_dimensions`]: struct.CairoRenderer.html#method.intrinsic_dimensions
 pub struct IntrinsicDimensions {
     /// `width` attribute of the `<svg>`, if present
     pub width: Option<Length>,
