@@ -78,6 +78,7 @@ use cssparser::{
     CowRcStr, DeclarationListParser, DeclarationParser, Parser, ParserInput, QualifiedRuleParser,
     RuleListParser, SourceLocation, ToCss, _cssparser_internal_to_lowercase,
 };
+use data_url::mime::Mime;
 use markup5ever::{namespace_url, ns, LocalName, Namespace, Prefix, QualName};
 use selectors::attr::{AttrSelectorOperation, CaseSensitivity, NamespaceConstraint};
 use selectors::matching::{ElementSelectorFlags, MatchingContext, MatchingMode, QuirksMode};
@@ -297,6 +298,7 @@ impl<'i> AtRuleParser<'i> for RuleParser {
 }
 
 /// Dummy type required by the SelectorImpl trait.
+#[allow(clippy::upper_case_acronyms)]
 #[derive(Clone, Debug, Eq, PartialEq)]
 pub struct NonTSPseudoClass;
 
@@ -696,10 +698,10 @@ impl Stylesheet {
             .and_then(|data| {
                 let BinaryData {
                     data: bytes,
-                    content_type,
+                    mime_type,
                 } = data;
 
-                if content_type.as_ref().map(String::as_ref) == Some("text/css") {
+                if is_text_css(&mime_type) {
                     Ok(bytes)
                 } else {
                     rsvg_log!("\"{}\" is not of type text/css; ignoring", aurl);
@@ -751,6 +753,10 @@ impl Stylesheet {
     }
 }
 
+fn is_text_css(mime_type: &Mime) -> bool {
+    mime_type.type_ == "text" && mime_type.subtype == "css"
+}
+
 /// Runs the CSS cascade on the specified tree from all the stylesheets
 pub fn cascade(
     root: &mut Node,
@@ -795,28 +801,13 @@ pub fn cascade(
 #[cfg(test)]
 mod tests {
     use super::*;
-    use gio;
-    use glib::{self, prelude::*};
     use selectors::Element;
 
     use crate::document::Document;
-    use crate::handle::LoadOptions;
-
-    fn load_document(input: &'static [u8]) -> Document {
-        let bytes = glib::Bytes::from_static(input);
-        let stream = gio::MemoryInputStream::new_from_bytes(&bytes);
-
-        Document::load_from_stream(
-            &LoadOptions::new(UrlResolver::new(None)),
-            &stream.upcast(),
-            None::<&gio::Cancellable>,
-        )
-        .unwrap()
-    }
 
     #[test]
     fn impl_element() {
-        let document = load_document(
+        let document = Document::load_from_bytes(
             br#"<?xml version="1.0" encoding="UTF-8"?>
 <svg xmlns="http://www.w3.org/2000/svg" id="a">
   <rect id="b" x="10" y="10" width="30" height="30"/>
