@@ -7,14 +7,13 @@ use crate::bbox::BoundingBox;
 use crate::css::{Origin, Stylesheet};
 use crate::document::{AcquiredNodes, Document, NodeId};
 use crate::dpi::Dpi;
-use crate::drawing_ctx::{draw_tree, DrawingMode, SavedCr, ViewParams};
+use crate::drawing_ctx::{draw_tree, with_saved_cr, DrawingMode, ViewParams};
 use crate::error::{DefsLookupErrorKind, LoadingError, RenderingError};
 use crate::length::*;
 use crate::node::{CascadedValues, Node, NodeBorrow};
 use crate::rect::Rect;
 use crate::structure::IntrinsicDimensions;
 use crate::url_resolver::{AllowedUrl, UrlResolver};
-use crate::util::check_cairo_context;
 
 /// Loading options for SVG documents.
 #[derive(Clone)]
@@ -161,7 +160,7 @@ impl Handle {
         let root = self.document.root();
 
         let target = cairo::ImageSurface::create(cairo::Format::Rgb24, 1, 1)?;
-        let cr = cairo::Context::new(&target);
+        let cr = cairo::Context::new(&target)?;
 
         let bbox = draw_tree(
             DrawingMode::LimitToStack { node, root },
@@ -247,27 +246,26 @@ impl Handle {
         dpi: Dpi,
         is_testing: bool,
     ) -> Result<(), RenderingError> {
-        check_cairo_context(cr)?;
+        cr.status()?;
 
         let node = self.get_node_or_root(id)?;
         let root = self.document.root();
 
         let viewport = Rect::from(*viewport);
 
-        let _saved_cr = SavedCr::new(&cr);
-
-        let res = draw_tree(
-            DrawingMode::LimitToStack { node, root },
-            cr,
-            viewport,
-            user_language,
-            dpi,
-            false,
-            is_testing,
-            &mut AcquiredNodes::new(&self.document),
-        );
-
-        res.map(|_bbox| ())
+        with_saved_cr(cr, || {
+            draw_tree(
+                DrawingMode::LimitToStack { node, root },
+                cr,
+                viewport,
+                user_language,
+                dpi,
+                false,
+                is_testing,
+                &mut AcquiredNodes::new(&self.document),
+            )
+            .map(|_bbox| ())
+        })
     }
 
     fn get_bbox_for_element(
@@ -278,7 +276,7 @@ impl Handle {
         is_testing: bool,
     ) -> Result<BoundingBox, RenderingError> {
         let target = cairo::ImageSurface::create(cairo::Format::Rgb24, 1, 1)?;
-        let cr = cairo::Context::new(&target);
+        let cr = cairo::Context::new(&target)?;
 
         let node = node.clone();
 
@@ -327,7 +325,7 @@ impl Handle {
         dpi: Dpi,
         is_testing: bool,
     ) -> Result<(), RenderingError> {
-        check_cairo_context(cr)?;
+        cr.status()?;
 
         let node = self.get_node_or_root(id)?;
 
@@ -346,27 +344,26 @@ impl Handle {
 
         // Render, transforming so element is at the new viewport's origin
 
-        let _saved_cr = SavedCr::new(&cr);
+        with_saved_cr(cr, || {
+            let factor = (element_viewport.width / ink_r.width())
+                .min(element_viewport.height / ink_r.height());
 
-        let factor =
-            (element_viewport.width / ink_r.width()).min(element_viewport.height / ink_r.height());
+            cr.translate(element_viewport.x, element_viewport.y);
+            cr.scale(factor, factor);
+            cr.translate(-ink_r.x0, -ink_r.y0);
 
-        cr.translate(element_viewport.x, element_viewport.y);
-        cr.scale(factor, factor);
-        cr.translate(-ink_r.x0, -ink_r.y0);
-
-        let res = draw_tree(
-            DrawingMode::OnlyNode(node),
-            &cr,
-            unit_rectangle(),
-            user_language,
-            dpi,
-            false,
-            is_testing,
-            &mut AcquiredNodes::new(&self.document),
-        );
-
-        res.map(|_bbox| ())
+            draw_tree(
+                DrawingMode::OnlyNode(node),
+                &cr,
+                unit_rectangle(),
+                user_language,
+                dpi,
+                false,
+                is_testing,
+                &mut AcquiredNodes::new(&self.document),
+            )
+            .map(|_bbox| ())
+        })
     }
 
     pub fn get_intrinsic_dimensions(&self) -> IntrinsicDimensions {
