@@ -1,15 +1,11 @@
 //! Tree nodes, the representation of SVG elements.
 //!
 //! Librsvg uses the [rctree crate][rctree] to represent the SVG tree of elements.
-//! Its [`Node`] struct provides a generic wrapper over nodes in a tree.
-//! Librsvg puts a [`NodeData`] as the type parameter of [`Node`].  For convenience,
-//! librsvg has a type alias `Node = Node<NodeData>`.
+//! Its [`rctree::Node`] struct provides a generic wrapper over nodes in a tree.
+//! Librsvg puts a [`NodeData`] as the type parameter of [`rctree::Node`].  For convenience,
+//! librsvg has a type alias [`Node`]` = rctree::Node<NodeData>`.
 //!
 //! Nodes are not constructed directly by callers;
-//!
-//! [rctree]: ../../rctree/index.html
-//! [`Node`]: ../../rctree/struct.Node.html
-//! [`NodeData`]: struct.NodeData.html
 
 use markup5ever::QualName;
 use std::cell::{Ref, RefMut};
@@ -20,18 +16,19 @@ use crate::document::AcquiredNodes;
 use crate::drawing_ctx::DrawingCtx;
 use crate::element::*;
 use crate::error::*;
+use crate::paint_server::PaintSource;
 use crate::properties::ComputedValues;
 use crate::text::Chars;
 use crate::xml::Attributes;
 
 /// Strong reference to an element in the SVG tree.
 ///
-/// See the [module documentation](index.html) for more information.
+/// See the [module documentation][self] for more information.
 pub type Node = rctree::Node<NodeData>;
 
 /// Weak reference to an element in the SVG tree.
 ///
-/// See the [module documentation](index.html) for more information.
+/// See the [module documentation][self] for more information.
 pub type WeakNode = rctree::WeakNode<NodeData>;
 
 /// Data for a single DOM node.
@@ -104,6 +101,8 @@ impl fmt::Display for NodeData {
 /// `&ComputedValues` whose fields you can access.
 pub struct CascadedValues<'a> {
     inner: CascadedInner<'a>,
+    pub context_stroke: Option<PaintSource>,
+    pub context_fill: Option<PaintSource>,
 }
 
 enum CascadedInner<'a> {
@@ -121,6 +120,8 @@ impl<'a> CascadedValues<'a> {
         match self.inner {
             CascadedInner::FromNode(_) => CascadedValues {
                 inner: CascadedInner::FromNode(node.borrow_element()),
+                context_fill: self.context_fill.clone(),
+                context_stroke: self.context_stroke.clone(),
             },
 
             CascadedInner::FromValues(ref v) => CascadedValues::new_from_values(node, &*v),
@@ -135,6 +136,8 @@ impl<'a> CascadedValues<'a> {
     pub fn new_from_node(node: &Node) -> CascadedValues<'_> {
         CascadedValues {
             inner: CascadedInner::FromNode(node.borrow_element()),
+            context_fill: None,
+            context_stroke: None,
         }
     }
 
@@ -151,6 +154,8 @@ impl<'a> CascadedValues<'a> {
 
         CascadedValues {
             inner: CascadedInner::FromValues(v),
+            context_fill: None,
+            context_stroke: None,
         }
     }
 
@@ -163,6 +168,13 @@ impl<'a> CascadedValues<'a> {
             CascadedInner::FromNode(ref e) => e.get_computed_values(),
             CascadedInner::FromValues(ref v) => &*v,
         }
+
+        // if values.fill == "context-fill" {
+        //     values.fill=self.context_fill
+        // }
+        // if values.stroke == "context-stroke" {
+        //     values.stroke=self.context_stroke
+        // }
     }
 }
 
