@@ -21,8 +21,7 @@ use crate::util;
 /// `ComputedValues`), or a paint server like a gradient or pattern which is referenced by
 /// a URL that points to a certain document node.
 ///
-/// Use [`PaintServer.resolve`](#method.resolve) to turn this into a
-/// [`PaintSource`](enum.PaintSource.html).
+/// Use [`PaintServer.resolve`](#method.resolve) to turn this into a [`PaintSource`].
 #[derive(Debug, Clone, PartialEq)]
 pub enum PaintServer {
     /// For example, `fill="none"`.
@@ -36,12 +35,19 @@ pub enum PaintServer {
 
     /// For example, `fill="blue"`.
     SolidColor(cssparser::Color),
+
+    /// For example, `fill="context-fill"`
+    ContextFill,
+
+    /// For example, `fill="context-stroke"`
+    ContextStroke,
 }
 
 /// Paint server with resolved references, with unnormalized lengths.
 ///
 /// Use [`PaintSource.to_user_space`](#method.to_user_space) to turn this into a
-/// [`UserSpacePaintSource`](enum.UserSpacePaintSource.html).
+/// [`UserSpacePaintSource`].
+#[derive(Clone)]
 pub enum PaintSource {
     None,
     Gradient(ResolvedGradient, Option<cssparser::RGBA>),
@@ -66,6 +72,16 @@ impl Parse for PaintServer {
             .is_ok()
         {
             Ok(PaintServer::None)
+        } else if parser
+            .try_parse(|i| i.expect_ident_matching("context-fill"))
+            .is_ok()
+        {
+            Ok(PaintServer::ContextFill)
+        } else if parser
+            .try_parse(|i| i.expect_ident_matching("context-stroke"))
+            .is_ok()
+        {
+            Ok(PaintServer::ContextStroke)
         } else if let Ok(url) = parser.try_parse(|i| i.expect_url()) {
             let loc = parser.current_source_location();
 
@@ -108,6 +124,8 @@ impl PaintServer {
         acquired_nodes: &mut AcquiredNodes<'_>,
         opacity: UnitInterval,
         current_color: cssparser::RGBA,
+        context_fill: Option<PaintSource>,
+        context_stroke: Option<PaintSource>,
     ) -> PaintSource {
         match self {
             PaintServer::Iri {
@@ -183,6 +201,22 @@ impl PaintServer {
                 PaintSource::SolidColor(resolve_color(color, opacity, current_color))
             }
 
+            PaintServer::ContextFill => {
+                if let Some(paint) = context_fill {
+                    paint
+                } else {
+                    PaintSource::None
+                }
+            }
+
+            PaintServer::ContextStroke => {
+                if let Some(paint) = context_stroke {
+                    paint
+                } else {
+                    PaintSource::None
+                }
+            }
+
             PaintServer::None => PaintSource::None,
         }
     }
@@ -247,6 +281,17 @@ pub fn resolve_color(
     cssparser::RGBA { alpha, ..rgba }
 }
 
+impl std::fmt::Debug for PaintSource {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> Result<(), std::fmt::Error> {
+        match *self {
+            PaintSource::None => f.write_str("PaintSource::None"),
+            PaintSource::Gradient(_, _) => f.write_str("PaintSource::Gradient"),
+            PaintSource::Pattern(_, _) => f.write_str("PaintSource::Pattern"),
+            PaintSource::SolidColor(_) => f.write_str("PaintSource::SolidColor"),
+        }
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
