@@ -10,10 +10,10 @@ use crate::drawing_ctx::{ClipMode, DrawingCtx, ViewParams};
 use crate::element::{Draw, ElementResult, SetAttributes};
 use crate::error::*;
 use crate::href::{is_href, set_href};
+use crate::layout::StackingContext;
 use crate::length::*;
 use crate::node::{CascadedValues, Node, NodeBorrow, NodeDraw};
 use crate::parsers::{Parse, ParseValue};
-use crate::properties::ComputedValues;
 use crate::rect::Rect;
 use crate::viewbox::*;
 use crate::xml::Attributes;
@@ -34,9 +34,17 @@ impl Draw for Group {
     ) -> Result<BoundingBox, RenderingError> {
         let values = cascaded.get();
 
-        draw_ctx.with_discrete_layer(node, acquired_nodes, values, clipping, &mut |an, dc| {
-            node.draw_children(an, cascaded, dc, clipping)
-        })
+        let elt = node.borrow_element();
+        let stacking_ctx = StackingContext::new(acquired_nodes, &elt, elt.get_transform(), values);
+
+        draw_ctx.with_discrete_layer(
+            &stacking_ctx,
+            acquired_nodes,
+            values,
+            clipping,
+            None,
+            &mut |an, dc| node.draw_children(an, cascaded, dc, clipping),
+        )
     }
 }
 
@@ -67,16 +75,26 @@ impl Draw for Switch {
     ) -> Result<BoundingBox, RenderingError> {
         let values = cascaded.get();
 
-        draw_ctx.with_discrete_layer(node, acquired_nodes, values, clipping, &mut |an, dc| {
-            if let Some(child) = node.children().filter(|c| c.is_element()).find(|c| {
-                let elt = c.borrow_element();
-                elt.get_cond() && !elt.is_in_error()
-            }) {
-                child.draw(an, &CascadedValues::new(cascaded, &child), dc, clipping)
-            } else {
-                Ok(dc.empty_bbox())
-            }
-        })
+        let elt = node.borrow_element();
+        let stacking_ctx = StackingContext::new(acquired_nodes, &elt, elt.get_transform(), values);
+
+        draw_ctx.with_discrete_layer(
+            &stacking_ctx,
+            acquired_nodes,
+            values,
+            clipping,
+            None,
+            &mut |an, dc| {
+                if let Some(child) = node.children().filter(|c| c.is_element()).find(|c| {
+                    let elt = c.borrow_element();
+                    elt.get_cond(&dc.user_language()) && !elt.is_in_error()
+                }) {
+                    child.draw(an, &CascadedValues::new(cascaded, &child), dc, clipping)
+                } else {
+                    Ok(dc.empty_bbox())
+                }
+            },
+        )
     }
 }
 
@@ -140,18 +158,18 @@ impl Svg {
         (w, h)
     }
 
-    fn get_viewport(&self, values: &ComputedValues, params: &ViewParams, outermost: bool) -> Rect {
+    fn get_viewport(&self, params: &NormalizeParams, outermost: bool) -> Rect {
         // x & y attributes have no effect on outermost svg
         // http://www.w3.org/TR/SVG/struct.html#SVGElement
         let (nx, ny) = if outermost {
             (0.0, 0.0)
         } else {
             let (x, y) = self.get_unnormalized_offset();
-            (x.normalize(values, &params), y.normalize(values, &params))
+            (x.to_user(params), y.to_user(params))
         };
 
         let (w, h) = self.get_unnormalized_size();
-        let (nw, nh) = (w.normalize(values, &params), h.normalize(values, &params));
+        let (nw, nh) = (w.to_user(params), h.to_user(params));
 
         Rect::new(nx, ny, nx + nw, ny + nh)
     }
@@ -164,7 +182,8 @@ impl Svg {
     ) -> Option<ViewParams> {
         let values = cascaded.get();
 
-        let params = draw_ctx.get_view_params();
+        let view_params = draw_ctx.get_view_params();
+        let params = NormalizeParams::new(values, &view_params);
 
         let has_parent = node.parent().is_some();
 
@@ -174,7 +193,7 @@ impl Svg {
             None
         };
 
-        let svg_viewport = self.get_viewport(values, &params, !has_parent);
+        let svg_viewport = self.get_viewport(&params, !has_parent);
 
         let is_measuring_toplevel_svg = !has_parent && draw_ctx.is_measuring();
 
@@ -233,10 +252,20 @@ impl Draw for Svg {
     ) -> Result<BoundingBox, RenderingError> {
         let values = cascaded.get();
 
-        draw_ctx.with_discrete_layer(node, acquired_nodes, values, clipping, &mut |an, dc| {
-            let _params = self.push_viewport(node, cascaded, dc);
-            node.draw_children(an, cascaded, dc, clipping)
-        })
+        let elt = node.borrow_element();
+        let stacking_ctx = StackingContext::new(acquired_nodes, &elt, elt.get_transform(), values);
+
+        draw_ctx.with_discrete_layer(
+            &stacking_ctx,
+            acquired_nodes,
+            values,
+            clipping,
+            None,
+            &mut |an, dc| {
+                let _params = self.push_viewport(node, cascaded, dc);
+                node.draw_children(an, cascaded, dc, clipping)
+            },
+        )
     }
 }
 
@@ -249,11 +278,11 @@ pub struct Use {
 }
 
 impl Use {
-    pub fn get_rect(&self, values: &ComputedValues, params: &ViewParams) -> Rect {
-        let x = self.x.normalize(values, &params);
-        let y = self.y.normalize(values, &params);
-        let w = self.width.normalize(values, &params);
-        let h = self.height.normalize(values, &params);
+    fn get_rect(&self, params: &NormalizeParams) -> Rect {
+        let x = self.x.to_user(params);
+        let y = self.y.to_user(params);
+        let w = self.width.to_user(params);
+        let h = self.height.to_user(params);
 
         Rect::new(x, y, x + w, y + h)
     }
@@ -302,7 +331,12 @@ impl Draw for Use {
         clipping: bool,
     ) -> Result<BoundingBox, RenderingError> {
         if let Some(link) = self.link.as_ref() {
-            draw_ctx.draw_from_use_node(node, acquired_nodes, cascaded, link, clipping)
+            let values = cascaded.get();
+            let view_params = draw_ctx.get_view_params();
+            let params = NormalizeParams::new(values, &view_params);
+            let rect = self.get_rect(&params);
+
+            draw_ctx.draw_from_use_node(node, acquired_nodes, values, rect, link, clipping)
         } else {
             Ok(draw_ctx.empty_bbox())
         }
@@ -409,11 +443,11 @@ impl Mask {
         CoordUnits::from(self.content_units)
     }
 
-    pub fn get_rect(&self, values: &ComputedValues, params: &ViewParams) -> Rect {
-        let x = self.x.normalize(&values, &params);
-        let y = self.y.normalize(&values, &params);
-        let w = self.width.normalize(&values, &params);
-        let h = self.height.normalize(&values, &params);
+    pub fn get_rect(&self, params: &NormalizeParams) -> Rect {
+        let x = self.x.to_user(params);
+        let y = self.y.to_user(params);
+        let w = self.width.to_user(params);
+        let h = self.height.to_user(params);
 
         Rect::new(x, y, x + w, y + h)
     }
@@ -469,13 +503,21 @@ impl Draw for Link {
         let cascaded = CascadedValues::new(cascaded, node);
         let values = cascaded.get();
 
-        draw_ctx.with_discrete_layer(node, acquired_nodes, values, clipping, &mut |an, dc| {
-            match self.link.as_ref() {
+        let elt = node.borrow_element();
+        let stacking_ctx = StackingContext::new(acquired_nodes, &elt, elt.get_transform(), values);
+
+        draw_ctx.with_discrete_layer(
+            &stacking_ctx,
+            acquired_nodes,
+            values,
+            clipping,
+            None,
+            &mut |an, dc| match self.link.as_ref() {
                 Some(l) if !l.is_empty() => {
                     dc.with_link_tag(l, &mut |dc| node.draw_children(an, &cascaded, dc, clipping))
                 }
                 _ => node.draw_children(an, &cascaded, dc, clipping),
-            }
-        })
+            },
+        )
     }
 }
