@@ -65,7 +65,7 @@ fn reference_test(path: &str) {
     .unwrap();
 
     {
-        let cr = cairo::Context::new(&surface);
+        let cr = cairo::Context::new(&surface).expect("Failed to create a cairo context");
         cr.translate(f64::from(FRAME_SIZE), f64::from(FRAME_SIZE));
         renderer
             .render_document(
@@ -107,9 +107,10 @@ fn extract_rectangle(
     h: i32,
 ) -> Result<cairo::ImageSurface, cairo::Error> {
     let dest = cairo::ImageSurface::create(cairo::Format::ARgb32, w, h)?;
-    let cr = cairo::Context::new(&dest);
-    cr.set_source_surface(&source, f64::from(-x), f64::from(-y));
-    cr.paint();
+    let cr = cairo::Context::new(&dest).expect("Failed to create a cairo context");
+    cr.set_source_surface(&source, f64::from(-x), f64::from(-y))
+        .unwrap();
+    cr.paint().unwrap();
     Ok(dest)
 }
 
@@ -241,6 +242,7 @@ test_compare_render_output!(
   
     <path d="M20,50 L80,50" marker-start="url(#marker)" marker-end="url(#marker)" stroke-width="10" stroke="black"/>
   </svg>"##,
+
     br##"<?xml version="1.0" encoding="UTF-8" standalone="no"?>
     <svg xmlns="http://www.w3.org/2000/svg" width="100" height="100">
       <path
@@ -256,4 +258,80 @@ test_compare_render_output!(
          x="20"
          y="45" fill="black"/>
     </svg>"##,
-  );
+);
+
+test_compare_render_output!(
+    marker_context_stroke_fill,
+    400,
+    400,
+    br##"<?xml version="1.0" encoding="UTF-8"?>
+    <svg xmlns="http://www.w3.org/2000/svg" width="400" height="400">
+      <style>
+        .path1 {
+          fill: none;
+          stroke-width: 4px;
+          marker: url(#marker1);
+        }
+    
+        .path2 {
+          fill: darkblue;
+          stroke: mediumseagreen;
+          stroke-width: 4px;
+          marker: url(#marker2);
+        }
+      </style>
+    
+      <path class="path1" d="M20,20 L200,20 L380,20" stroke="lime"/>
+    
+      <path class="path2" d="M20,40 h360 v320 h-360 v-320 Z"/>
+    
+      <marker id="marker1" markerWidth="12" markerHeight="12" refX="6" refY="6"
+              markerUnits="userSpaceOnUse">
+        <circle cx="6" cy="6" r="3"
+                fill="white" stroke="context-stroke" stroke-width="2"/>
+      </marker>
+    
+      <marker id="marker2" markerWidth="12" markerHeight="12" refX="6" refY="6"
+              markerUnits="userSpaceOnUse">
+        <!-- Note that here the paint is reversed:
+             fill=context-stroke,
+             stroke=context-fill 
+        -->
+        <circle cx="6" cy="6" r="3"
+                fill="context-stroke" stroke="context-fill" stroke-width="2"/>
+      </marker>
+    </svg>"##,
+    br##"<?xml version="1.0" encoding="UTF-8"?>
+    <svg xmlns="http://www.w3.org/2000/svg" width="400" height="400">
+      <path d="M20,20 L200,20 L380,20" stroke="lime" stroke-width="4"/>
+      <circle cx="20" cy="20" r="3" stroke-width="2" fill="white" stroke="lime"/>
+      <circle cx="200" cy="20" r="3" stroke-width="2" fill="white" stroke="lime"/>
+      <circle cx="380" cy="20" r="3" stroke-width="2" fill="white" stroke="lime"/>
+    
+      <path class="path2" d="M20,40 h360 v320 h-360 v-320 Z" fill="darkblue"
+            stroke="mediumseagreen" stroke-width="4"/>
+      <circle cx="20"  cy="40"  r="3" fill="mediumseagreen" stroke="darkblue" stroke-width="2"/>
+      <circle cx="20"  cy="40"  r="3" fill="mediumseagreen" stroke="darkblue" stroke-width="2"/>
+      <circle cx="380" cy="40"  r="3" fill="mediumseagreen" stroke="darkblue" stroke-width="2"/>
+      <circle cx="380" cy="360" r="3" fill="mediumseagreen" stroke="darkblue" stroke-width="2"/>
+      <circle cx="20"  cy="360" r="3" fill="mediumseagreen" stroke="darkblue" stroke-width="2"/>
+      <circle cx="20"  cy="40"  r="3" fill="mediumseagreen" stroke="darkblue" stroke-width="2"/>
+    </svg>
+    "##,
+);
+
+test_compare_render_output!(
+    image_auto_width_height,
+    30,
+    30,
+    br##"<?xml version="1.0" encoding="UTF-8"?>
+    <svg xmlns="http://www.w3.org/2000/svg" width="30" height="30">
+      <image
+        href="data:;base64,iVBORw0KGgoAAAANSUhEUgAAAAoAAAAKCAIAAAACUFjqAAAAFElEQVQY02Nk+M+ABzAxMIxKYwIAQC0BEwZFOw4AAAAASUVORK5CYII="
+        x="10" y="10"/>
+    </svg>"##,
+    br##"<?xml version="1.0" encoding="UTF-8"?>
+    <svg xmlns="http://www.w3.org/2000/svg" width="30" height="30">
+      <rect x="10" y="10" width="10" height="10" fill="lime"/>
+    </svg>"##,
+);
